#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Malibu_Mb.h>

using namespace Innovative;
using namespace std;

// Set to zero while adding event handlers to work-around
// QtCreator bug which adds slots here instead of form header
#if 1
//===========================================================================
//  CLASS UserInteface  -- Interface for callbacks to UI from App Logic
//===========================================================================

class UserInterface : public IUserInterface
{
public:
    // Ctor
    UserInterface(MainWindow* form)
        : Form(form)
        {  }

    // Methods
    virtual void  Log(const std::string & a_string)
        {  Form->Log(a_string);  }
    virtual void  OnProgress(int percent)
        {  Form->OnProgress(percent);  }
    virtual void OnLoadMessage(const std::string & Message)
        {  Form->OnLoadMessage(Message);  }
    virtual void OnLoadComplete(unsigned int result)
        {  Form->OnLoadComplete(result);  }
    virtual void OnElapsed(const std::string & msg)
        {  Form->Clock(msg);  }
    virtual void OnRevision(const std::string & msg)
        {  Form->Revision(msg);  }

private:
    // Data
    MainWindow *       Form;
};

#endif

//===========================================================================
//  CLASS MainWindow  -- Form methods
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::MainWindow() --  Ctor
//---------------------------------------------------------------------------

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    UI = new UserInterface(this);
    Io = new ApplicationIo(UI);

    RevisionBar = new QStatusBar(ui->RevisionBarFrame);
    RevisionBar->setSizeGripEnabled(false);
    Revision("Revision:");
    Clock("Elapsed:");

    int Count = Io->BoardCount();
    if (!Count)
        {
        ControlEnabled(ui->TargetComboBox, false);
        Log("No devices detected!");
        }
    else
        {
        for (int i = 0; i < Count; ++i)
            AddToControl(ui->TargetComboBox, IntToString(i));

        ControlEnabled(ui->LoadButton, true);
        ControlEnabled(ui->TargetComboBox, true);
        ToControl(ui->TargetComboBox, 0);
        }

    SetSettings();
    stringstream msg;
    msg << Io->BoardName() << " Eeprom Programmer";
    ToCaption(this, msg.str());

//    ui->tabWidget->ccurrentIndex(0);

    if (!Io->Settings.Debug)
        {
        ControlVisible(ui->DebugTab, false);
        ToControl(ui->DebugLevelComboBox, 0);
        ToControl(ui->LogFileCheckBox, false);
        ToControl(ui->DisplayXsvfCheckBox, false);
        }

    Status("Ready");
}

//---------------------------------------------------------------------------
//  MainWindow::~MainWindow() --  Dtor
//---------------------------------------------------------------------------

MainWindow::~MainWindow()
{
    GetSettings();

    delete Io;
    Io = 0;

    delete UI;
    UI = 0;

    delete ui;
}


void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//---------------------------------------------------------------------------
//  MainWindow::on_OpenButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_OpenButton_clicked()
{
    try
        {
        Io->Close();
        int target;
        FromControl(ui->TargetComboBox, target);
        Io->Open(target);
        ControlEnabled(ui->OpenButton, false);
        ControlEnabled(ui->CloseButton, true);
        ControlEnabled(ui->TargetComboBox, false);
        }
    catch(...)
        {
        Log("Unable to open device driver.");
        Log("Check Windows Device Manager to ensure that board enumerated.");
        return;
        }

    Status("Ready");
}

//---------------------------------------------------------------------------
//  MainWindow::on_CloseButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_CloseButton_clicked()
{
    Io->Close();
    ControlEnabled(ui->OpenButton, true);
    ControlEnabled(ui->CloseButton, false);
    ControlEnabled(ui->TargetComboBox, true);
}

//---------------------------------------------------------------------------
//  MainWindow::on_LoadButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_LoadButton_clicked()
{
    GetSettings();

    ControlEnabled(ui->LoadButton, false);
    ControlEnabled(ui->BrowseFileButton, false);

    Status("Parsing logic file...");
    Io->Load();

}

//---------------------------------------------------------------------------
//  MainWindow::on_BrowseFileButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_BrowseFileButton_clicked()
{
    Status("Browsing for logic file...");

    SelectFileDialog(ui->FileEdit, "Xsvf Files *.xsvf *.*");

    Status("Ready");
}

//---------------------------------------------------------------------------
//  MainWindow::Log() --  Append text to event log
//---------------------------------------------------------------------------

void MainWindow::Log(const std::string & s)
{
    AddToControl(ui->LogMemo, s);
}

//---------------------------------------------------------------------------
//  MainWindow::BlinkClick() --
//---------------------------------------------------------------------------

void MainWindow::EnableControls()
{
/*
   ControlEnabled(ui->OpenButton, !Io->IsOpen());
   ControlEnabled(ui->CloseButton, Io->IsOpen());
   ControlEnabled(ui->LedOn, Io->IsOpen());
   ControlEnabled(ui->LedOff, Io->IsOpen());
   ControlEnabled(ui->BlinkButton, Io->IsOpen());
   ControlEnabled(ui->TargetComboBox, !Io->IsOpen());
*/
}

//---------------------------------------------------------------------------
//  MainWindow::Status() --
//---------------------------------------------------------------------------

void MainWindow::Status(const std::string & txt)
{
    ToControl(ui->StatusBar, 0, txt);
}

//---------------------------------------------------------------------------
//  MainWindow::Clock() --
//---------------------------------------------------------------------------

void MainWindow::Clock(const std::string & txt)
{
    ToControl(ui->StatusBar, 1, txt);
}

//---------------------------------------------------------------------------
//  MainWindow::Clock() --
//---------------------------------------------------------------------------

void MainWindow::Revision(const std::string & txt)
{
    ToControl(RevisionBar, 0, txt);
}

//------------------------------------------------------------------------------
//  MainWindow::OnProgress() --
//------------------------------------------------------------------------------

void MainWindow::OnProgress(int percent)
{
    ToControl(ui->ProgressBar, percent);
}

//------------------------------------------------------------------------------
//  MainWindow::OnLoadMessage() --
//------------------------------------------------------------------------------

void MainWindow::OnLoadMessage(const std::string & Message)
{
    Log(Message);
}

//------------------------------------------------------------------------------
//  MainWindow::OnLoadComplete() --
//------------------------------------------------------------------------------

void MainWindow::OnLoadComplete(unsigned int Result)
{
    if (Result == 0)
        {
        ToControl(ui->ProgressBar, 100);
        Log("Xsvf loaded successfully");
        }
    else
        {
        Log("Xsvf load failed with code: " + IntToString(Result));
        }

    ControlEnabled(ui->LoadButton, true);
    ControlEnabled(ui->BrowseFileButton, true);

    Status("Ready");
}

//---------------------------------------------------------------------------
//  MainWindow::GetSettings() --  Form controls to options object
//---------------------------------------------------------------------------

void MainWindow::GetSettings()
{
    FromControl(ui->TargetComboBox, Io->Settings.Target);
    FromControl(ui->DebugLevelComboBox, Io->Settings.DebugLevel);
    FromControl(ui->LogFileCheckBox, Io->Settings.LogFile);
    FromControl(ui->DisplayXsvfCheckBox, Io->Settings.LogMessages);
    FromControl(ui->FileEdit, Io->Settings.XsvfFile);
}

//---------------------------------------------------------------------------
//  MainWindow::SetSettings() --  Settings object to form controls
//---------------------------------------------------------------------------

void MainWindow::SetSettings()
{
    ToControl(ui->TargetComboBox, Io->Settings.Target);
    ToControl(ui->DebugLevelComboBox, Io->Settings.DebugLevel);
    ToControl(ui->LogFileCheckBox, Io->Settings.LogFile);
    ToControl(ui->DisplayXsvfCheckBox, Io->Settings.LogMessages);
    ToControl(ui->FileEdit, Io->Settings.XsvfFile);
}

//---------------------------------------------------------------------------
