#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Malibu_Mb.h>

using namespace Innovative;
using namespace std;

// Set to zero while adding event handlers to work-around
// QtCreator bug which adds slots here instead of form header
#if 1
//===========================================================================
//  CLASS GuiInterface  -- Interface for callbacks to UI from App Logic
//===========================================================================

class UserInterface : public IUserInterface
{
public:
    // Ctor
    UserInterface(MainWindow* form)
        : Form(form)
        {  }

    // Methods
    virtual void  Log(const std::string & a_string)
        {  Form->Log(a_string);  }

private:
    // Data
    MainWindow *       Form;
};
#endif

//===========================================================================
//  CLASS MainWindow  -- Form methods
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::MainWindow() --  Ctor
//---------------------------------------------------------------------------

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    UI = new UserInterface(this);
    Io = new ApplicationIo(UI);

    int Count = Io->BoardCount();

    // Quick License Check
    Io->LicenseCheck();

    if (!Count)
        {
        ControlEnabled(ui->TargetComboBox, false);
        Log("No devices detected!");
        }
    else
        {
        for (int i = 0; i < Count; ++i)
            AddToControl(ui->TargetComboBox, IntToString(i));

        ControlEnabled(ui->TargetComboBox, true);
        ToControl(ui->TargetComboBox, 0);
        }

    stringstream msg;
    msg << Io->BoardName() << " Finder";
    ToCaption(this, msg.str());
}

//---------------------------------------------------------------------------
//  MainWindow::~MainWindow() --  Dtor
//---------------------------------------------------------------------------

MainWindow::~MainWindow()
{
    delete Io;
    Io = 0;

    delete UI;
    UI = 0;

    delete ui;
}


void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//---------------------------------------------------------------------------
//  MainWindow::on_OpenButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_OpenButton_clicked()
{
    int target;
    FromControl(ui->TargetComboBox, target);
    Io->Open(target);

    EnableControls();
}

//---------------------------------------------------------------------------
//  MainWindow::on_CloseButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_CloseButton_clicked()
{
    Io->Close();

    EnableControls();
}

//---------------------------------------------------------------------------
//  MainWindow::Log() --  Append text to event log
//---------------------------------------------------------------------------

void MainWindow::Log(const std::string & s)
{
    AddToControl(ui->LogMemo, s);
}

//---------------------------------------------------------------------------
//  MainWindow::BlinkClick() --
//---------------------------------------------------------------------------

void MainWindow::EnableControls()
{
   ControlEnabled(ui->OpenButton, !Io->IsOpen());
   ControlEnabled(ui->CloseButton, Io->IsOpen());
   ControlEnabled(ui->LedOn, Io->IsOpen());
   ControlEnabled(ui->LedOff, Io->IsOpen());
   ControlEnabled(ui->BlinkButton, Io->IsOpen());
   ControlEnabled(ui->TargetComboBox, !Io->IsOpen());
}

//---------------------------------------------------------------------------
//  MainWindow::on_LedOn_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_LedOn_clicked()
{
    Io->Led(true);
}

//---------------------------------------------------------------------------
//  MainWindow::on_LedOff_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_LedOff_clicked()
{
    Io->Led(false);
}

//---------------------------------------------------------------------------
//  MainWindow::on_BlinkButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_BlinkButton_clicked()
{
    //
    // If Blinking, stop. If not, start
    if (Io->Blink())
        {
        // Now Blinking
        Io->Blink(false);
        ControlEnabled(ui->TargetComboBox, true);
        ControlEnabled(ui->LedOn, true);
        ControlEnabled(ui->LedOff, true);
        Io->Led(true);
        ToControl(ui->BlinkButton, "Blink");
        }
    else
        {
        // Not Blinking
        Io->Blink(true);
        ControlEnabled(ui->TargetComboBox, false);
        ControlEnabled(ui->LedOn, false);
        ControlEnabled(ui->LedOff, false);
        ToControl(ui->BlinkButton, "Stop");
        }
}
