//---------------------------------------------------------------------------

#ifndef ApplicationIoH
#define ApplicationIoH

#include <IniFile_Mb.h>
#include "Target.h"
#include "SoftwareTimer_Mb.h"

//==============================================================================
//  CLASS IUserInterface - Interface to communicate back to UI
//==============================================================================

class IUserInterface
{
public:
    virtual ~IUserInterface()  {  }
    virtual void  Log(const std::string & a_string) = 0;
    virtual void  OnProgress(int percent) = 0;
    virtual void  OnParseComplete() = 0;
    virtual void  OnLoadError(const std::string & Message) = 0;
    virtual void  OnLoadComplete() = 0;
    virtual void  OnRevision(const std::string & Message) = 0;
};

//==============================================================================
//  CLASS ApplicationSettings - Remember Settings between runs
//==============================================================================

class ApplicationSettings
{
public:
    // Ctor
    ApplicationSettings();
	~ApplicationSettings();

    //  Data
    int             Target;
    bool            Debug;
    bool            Auto;
    std::string     ImageFile;

    std::string Path()
        {  return FPath;  }

private:
    // Data
    Innovative::IniFile     * Ini;
    std::string             FPath;
};


//==============================================================================
//  CLASS ApplicationIo  -- Hardware Access and Application Io Class
//==============================================================================

class ApplicationIo
{
public:
	//
    //  Member Functions
    ApplicationIo(IUserInterface * ui);
    ~ApplicationIo();

    void   Open();
    bool   IsOpen() const
        {  return FOpen;  }
    void   Close();
    void   Sleep(unsigned int ms);

    void   ConfigureFpga(const std::string & exo_file);

    size_t        BoardCount();
    std::string   BoardName();

    Innovative::FpgaLogicVersionInfoIntf &  LogicVersion();

    ApplicationSettings     Settings;

private:
    //
    //  Member Data
    IUserInterface *            UI;
    PmcModule                   Baseboard;
    bool                        FOpen;
    Innovative::SoftwareTimer   Timer;

    // Event Handlers
    void        HandleParseProgress(Innovative::ProcessProgressEvent & event);
    void        HandleParseComplete(Innovative::ProcessCompletionEvent & event);
    void        HandleLoadProgress(Innovative::ProcessProgressEvent & event);
    void        HandleLoadError(Innovative::ProcessStatusEvent & event);
    void        HandleLoadComplete(Innovative::ProcessCompletionEvent & event);
    void        HandleTimer(OpenWire::NotifyEvent & /*Event*/);

    void        DisplayLogicVersion();
};


#endif
