#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <ApplicationIo.h>
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Framework/GuiLoader_Mb.h>
#include <sstream>

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::MainWindow *ui;

    friend class UserInterface;

private slots:
    void on_ClockSourceComboBox_currentIndexChanged(int index);
    void on_AssButton_clicked();
    void on_BssButton_clicked();
    void on_LogicLoadConfigBtn_clicked();
    void on_LogicLoadBrowseBtn_clicked();
    void on_FiclCommandEdit_returnPressed();
    void on_FiclCommandEdit_textEdited(QString );
    void on_DeviceWriteButton_clicked();
    void on_DeviceReadButton_clicked();
    void on_PortWriteButton_clicked();
    void on_PortReadButton_clicked();
    void on_LogicWriteButton_clicked();
    void on_LogicReadButton_clicked();
    void on_SoftwareAlertButton_clicked();
    void on_DebugScriptExecuteButton_clicked();
    void on_DebugScriptBrowseButton_clicked();
    void on_StoreToRomButton_clicked();
    void on_LoadFromRomButton_clicked();
    void on_StreamStopBtn_clicked();
    void on_StreamStartBtn_clicked();
    void on_CloseBtn_clicked();
    void on_OpenBtn_clicked();

private: // User declarations
    IUserInterface * UI;
    ApplicationIo  * Io;
    QStatusBar *     LogicStatusBar;
    QStatusBar *     PerformanceStatusBar;
    QStatusBar *     DioStatusBar;

    vector<string>  BoardNames;

    void  SetSettings();
    void  GetSettings();
    void  Log(const std::string & a_string);
    void  Status(const std::string & a_string);
    void  LogicStatus(const std::string & a_string);
    void  PeriodicStatus();
    void  ClockFrequency(float value);
    void  AfterStreamStop();
    void  UpdateLogicLoadProgress(int percent);
    void  AdjustEnables();

    // Data
    std::string  WaveFile;

};

//===========================================================================
//  CLASS UserInteface     Concrete user interface.
//===========================================================================

class UserInterface : public IUserInterface
{
public:
    UserInterface(MainWindow * form)
        : Form(form)
    { }

    void  UpdateLogicLoadProgress(int percent)
        {  Form->UpdateLogicLoadProgress(percent);  }
    virtual void  Log(const std::string & a_string)
        {  Form->Log(a_string);  }
    virtual void  Status(const std::string & a_string)
        {  Form->Status(a_string);  }
    virtual void  GetSettings()
        {  Form->GetSettings();  }
    virtual void  SetSettings()
        {  Form->SetSettings();  }
    virtual void  AfterStreamStop()
        {  Form->AfterStreamStop();  }
    virtual void  Warning(const std::string & a_string)
        {  Form->Status(a_string);  }
    virtual void  PeriodicStatus()
        {  Form->PeriodicStatus();  }
    virtual void  ClockFrequency(float freq)
        {  Form->ClockFrequency(freq);  }
    virtual void  LogicVersionStatus(const std::string & txt)
        {  Form->Status(txt);  }

protected:
    MainWindow *  Form;
};



#endif // MAINWINDOW_H
